package graphs;

import java.util.*;

/**
 * Trieda reprezentujuca uzol zakoreneneho stromu, v ktorom je ulozena hodnota typu T.
 */
public class Node<T> {
    private T value;
    private List<Node<T>> children;

    /**
     * Konstruktor, ktory vytvori uzol, v ktorom je ulozena hodnota value a ktory nema ziadne deti.
     * @param value Hodnota typu T, ktora bude vo vytvorenom uzle ulozena.
     */
    public Node(T value) {
        this.value = value;
        this.children = new ArrayList<>();
    }

    /**
     * Konstruktor, ktory vytvori uzol uchovavajuci hodnotu value s detmi podla zoznamu children.
     * @param value Hodnota typu T, ktora bude vo vytvorenom uzle ulozena.
     * @param children Zoznam deti vytvaraneho uzlu.
     */
    public Node(T value, List<Node<T>> children) {
        this(value);
        this.children.addAll(children);
    }

    /**
     * Metoda, ktora vrati hodnotu ulozenu v danom uzle.
     * @return Hodnota typu T ulozena v uzle.
     */
    public T getValue() {
        return value;
    }

    /**
     * Metoda, ktora vrati nemodifikovatelny pohlad na zoznam deti uzlu, pre ktory je tato metoda volana.
     * @return Nemodifikovatelny pohlad na zoznam deti uzlu.
     */
    public List<Node<T>> getChildren() {
        return Collections.unmodifiableList(children);
    }

    /**
     * Metoda, ktora prida uzol node na koniec zoznamu deti uzlu, pre ktory je tato metoda volana.
     * @param node Uzol pridavany na koniec zoznamu deti.
     */
    public void addChild(Node<T> node) {
        children.add(node);
    }

    /**
     * Metoda, ktora vrati textovu reprezentaciu podstromu zakoreneneho v uzle, pre ktory je tato metoda volana.
     * @return Textova reprezentacia prislusneho podstromu.
     */
    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append(value);
        for (Node node : getChildren()) {
            builder.append(" ( ");
            builder.append(node);
            builder.append(" )");
        }
        return builder.toString();
    }

}
